package eu.jonahbauer.wizard.core.machine;

import eu.jonahbauer.wizard.core.machine.states.GameData;
import eu.jonahbauer.wizard.core.machine.states.GameState;
import eu.jonahbauer.wizard.core.machine.states.game.Starting;
import eu.jonahbauer.wizard.core.messages.Observer;
import eu.jonahbauer.wizard.core.messages.observer.ObserverMessage;
import eu.jonahbauer.wizard.core.messages.observer.StateMessage;
import eu.jonahbauer.wizard.core.messages.player.PlayerMessage;
import eu.jonahbauer.wizard.core.model.Configuration;
import lombok.Getter;

import java.util.List;
import java.util.UUID;

import static eu.jonahbauer.wizard.core.machine.states.GameData.PLAYERS;

public final class Game extends Context<GameState, Game> {
    @Getter
    private final Configuration config;
    private final Observer observer;

    public Game(Configuration config, Observer observer) {
        this.config = config;
        this.observer = observer;
    }

    public void start(List<UUID> players) {
        start(new Starting(new GameData().with(PLAYERS, List.copyOf(players))));
    }

    public void resume(GameState state) {
        start(state);
    }

    public GameState stop() {
        GameState state = this.state;
        if (state != null) {
            finish();
            return state;
        }
        return null;
    }

    public void onMessage(UUID player, PlayerMessage message) {
        lock.lock();
        try {
            state.onMessage(this, player, message);
        } catch (IllegalStateException | IllegalArgumentException e) {
            throw e;
        } catch (Throwable t) {
            handleError(t);
        } finally {
            lock.unlock();
        }
    }

    @Override
    protected void onTransition(GameState from, GameState to) {
        notify(new StateMessage(to != null ? to.getClass() : null));
    }

    public void notify(ObserverMessage message) {
        try {
            observer.notify(message);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void notify(UUID player, ObserverMessage message) {
        try {
            observer.notify(player, message);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
