package eu.jonahbauer.wizard.core.machine.states.round;

import eu.jonahbauer.wizard.core.machine.states.GameData;
import eu.jonahbauer.wizard.core.machine.Game;
import eu.jonahbauer.wizard.core.messages.observer.TrumpMessage;
import eu.jonahbauer.wizard.core.messages.observer.UserInputMessage;
import eu.jonahbauer.wizard.core.messages.player.PickTrumpMessage;
import eu.jonahbauer.wizard.core.messages.player.PlayerMessage;
import eu.jonahbauer.wizard.core.model.Card;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.VisibleForTesting;

import java.util.Arrays;
import java.util.Map;
import java.util.UUID;

import static eu.jonahbauer.wizard.core.machine.states.GameData.*;
import static eu.jonahbauer.wizard.core.messages.observer.UserInputMessage.Action.PICK_TRUMP;

public final class DeterminingTrump extends RoundState {
    public DeterminingTrump(GameData data) {
        super(data.require(TRUMP_CARD).requireEach(PLAYERS, HANDS));
    }

    @Override
    public void onEnter(Game game) {
        Card trumpCard = getData().get(TRUMP_CARD);
        Card.Suit trumpSuit = trumpCard != null ? trumpCard.getTrumpSuit() : Card.Suit.NONE;
        if (trumpSuit == null) {
            game.notify(new TrumpMessage(trumpCard, null));
            game.notify(new UserInputMessage(getDealer(), PICK_TRUMP, getTimeout(game, true)));
            timeout(game);
        } else {
            transition(game, trumpSuit);
        }
    }

    @Override
    public void onTimeout(Game game) {
        Card.Suit[] suits = new Card.Suit[]{Card.Suit.BLUE, Card.Suit.GREEN, Card.Suit.RED, Card.Suit.YELLOW};
        transition(game, suits[(int) (Math.random() * suits.length)]);
    }

    @Override
    public void onMessage(Game game, UUID player, PlayerMessage message) {
        if (getDealer().equals(player) && message instanceof PickTrumpMessage trumpMessage) {
            checkTrumpSuit(trumpMessage.getTrumpSuit());
            transition(game, trumpMessage.getTrumpSuit());
        } else {
            super.onMessage(game, player, message);
        }
    }

    @VisibleForTesting
    void checkTrumpSuit(Card.Suit suit) {
        Card.Suit[] suits = new Card.Suit[]{Card.Suit.BLUE, Card.Suit.GREEN, Card.Suit.RED, Card.Suit.YELLOW};
        for (Card.Suit s : suits) {
            if (s == suit) return;
        }
        throw new IllegalArgumentException("Trump suit must be one of " + Arrays.toString(suits) + ".");
    }

    private void transition(Game game, @NotNull Card.Suit trumpSuit) {
        game.notify(new TrumpMessage(getData().get(TRUMP_CARD), trumpSuit));
        transition(game, new Predicting(getData().with(
                TRUMP_SUIT, trumpSuit,
                PREDICTIONS, Map.of(),
                CURRENT_PLAYER, getNextPlayer(getDealer())
        )));
    }
}
