package eu.jonahbauer.wizard.core.model;

import lombok.experimental.UtilityClass;
import org.jetbrains.annotations.Contract;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.NoSuchElementException;

@UtilityClass
public class Configurations {
    private static final Map<String, Configuration> CONFIGURATIONS = new HashMap<>();

    public static final Configuration DEFAULT = register("default", new Configuration(
            Cards.DEFAULT,
            true,
            10 * 60 * 1000
    ));

    @Contract("_,_ -> param2")
    private static Configuration register(@NotNull String name, @NotNull Configuration configuration) {
        if (CONFIGURATIONS.putIfAbsent(name.toLowerCase(Locale.ROOT), configuration) != null) {
            throw new IllegalArgumentException("Name already taken.");
        }
        return configuration;
    }

    @NotNull
    public static Configuration get(@NotNull String name) {
        var out = CONFIGURATIONS.get(name.toLowerCase(Locale.ROOT));
        if (out == null) throw new NoSuchElementException("Configuration with name '" + name + "' does not exist.");
        return out;
    }
}
