package eu.jonahbauer.wizard.core.machine.states.trick;

import eu.jonahbauer.wizard.core.machine.states.GameData;
import eu.jonahbauer.wizard.core.model.Card;
import eu.jonahbauer.wizard.core.util.Pair;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.UUID;

import static eu.jonahbauer.wizard.core.machine.states.GameData.STACK;
import static eu.jonahbauer.wizard.core.machine.states.GameData.TRUMP_SUIT;
import static org.junit.jupiter.api.Assertions.assertNotNull;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class FinishingTrickTest {
    private void performTest(List<Pair<UUID, Card>> stack, Card.Suit trumpSuit, Card.Suit trickSuit) {
        FinishingTrick state = mock(FinishingTrick.class);
        when(state.getWinner()).thenCallRealMethod();
        when(state.get(STACK)).thenReturn(stack);
        when(state.get(TRUMP_SUIT)).thenReturn(trumpSuit);
        when(state.getTrickSuit()).thenReturn(trickSuit);

        assertNotNull(state.getWinner());
    }

    @Test
    public void getWinner_ReturnsFirstWizard() {
        List<Pair<UUID, Card>> stack = List.of(
                Pair.of(null, Card.RED_1),
                Pair.of(null, Card.YELLOW_1),
                Pair.of(UUID.randomUUID(), Card.BLUE_WIZARD),
                Pair.of(null, Card.BLUE_JESTER),
                Pair.of(null, Card.GREEN_11)
        );

        performTest(stack, Card.Suit.YELLOW, Card.Suit.RED);
    }

    @Test
    public void getWinner_ReturnsHighestTrump_IfNoWizard() {
        List<Pair<UUID, Card>> stack = List.of(
                Pair.of(null, Card.RED_1),
                Pair.of(null, Card.YELLOW_1),
                Pair.of(null, Card.BLUE_JESTER),
                Pair.of(UUID.randomUUID(), Card.YELLOW_13),
                Pair.of(null, Card.GREEN_11)
        );

        performTest(stack, Card.Suit.YELLOW, Card.Suit.RED);
    }

    @Test
    public void getWinner_ReturnsHighestTrickSuit_IfNeitherWizardNorTrump() {
        List<Pair<UUID, Card>> stack = List.of(
                Pair.of(null, Card.YELLOW_1),
                Pair.of(null, Card.BLUE_JESTER),
                Pair.of(UUID.randomUUID(), Card.RED_1),
                Pair.of(null, Card.YELLOW_13),
                Pair.of(null, Card.GREEN_11)
        );

        performTest(stack, Card.Suit.BLUE, Card.Suit.RED);
    }

    @Test
    public void getWinner_ReturnsFirstJester_IfOnlyJester() {
        List<Pair<UUID, Card>> stack = List.of(
                Pair.of(UUID.randomUUID(), Card.GREEN_JESTER),
                Pair.of(null, Card.BLUE_JESTER),
                Pair.of(null, Card.RED_JESTER),
                Pair.of(null, Card.YELLOW_JESTER)
        );

        performTest(stack, Card.Suit.NONE, Card.Suit.NONE);
    }
}
