package eu.jonahbauer.wizard.core.machine.states.trick;

import eu.jonahbauer.wizard.core.model.Card;
import eu.jonahbauer.wizard.core.util.Pair;
import org.junit.jupiter.api.Test;

import java.util.List;
import java.util.UUID;

import static eu.jonahbauer.wizard.core.machine.states.GameData.STACK;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

public class TrickStateTest {
    @Test
    public void getTrickSuit_ReturnsNone_IfEmpty() {
        List<Pair<UUID, Card>> stack = List.of();

        TrickState trickState = mock(TrickState.class);
        when(trickState.getTrickSuit()).thenCallRealMethod();
        when(trickState.get(STACK)).thenReturn(stack);

        assertEquals(trickState.getTrickSuit(), Card.Suit.NONE);
    }

    @Test
    public void getTrickSuit_ReturnsColor() {
        List<Pair<UUID, Card>> stack = List.of(
                Pair.of(null, Card.RED_1),
                Pair.of(null, Card.YELLOW_1),
                Pair.of(null, Card.BLUE_WIZARD),
                Pair.of(null, Card.BLUE_JESTER),
                Pair.of(null, Card.GREEN_11)
        );

        TrickState trickState = mock(TrickState.class);
        when(trickState.getTrickSuit()).thenCallRealMethod();
        when(trickState.get(STACK)).thenReturn(stack);

        assertEquals(trickState.getTrickSuit(), Card.Suit.RED);
    }

    @Test
    public void getTickSuit_ReturnsNone_IfFirstCardIsWizard() {
        List<Pair<UUID, Card>> stack = List.of(
                Pair.of(null, Card.BLUE_WIZARD),
                Pair.of(null, Card.RED_1),
                Pair.of(null, Card.YELLOW_1),
                Pair.of(null, Card.BLUE_JESTER),
                Pair.of(null, Card.GREEN_11)
        );

        TrickState trickState = mock(TrickState.class);
        when(trickState.getTrickSuit()).thenCallRealMethod();
        when(trickState.get(STACK)).thenReturn(stack);

        assertEquals(trickState.getTrickSuit(), Card.Suit.NONE);
    }

    @Test
    public void getTrickSuit_IgnoresJesters() {
        List<Pair<UUID, Card>> stack = List.of(
                Pair.of(null, Card.BLUE_JESTER),
                Pair.of(null, Card.BLUE_WIZARD),
                Pair.of(null, Card.RED_1),
                Pair.of(null, Card.YELLOW_1),
                Pair.of(null, Card.GREEN_11)
        );

        TrickState trickState = mock(TrickState.class);
        when(trickState.getTrickSuit()).thenCallRealMethod();
        when(trickState.get(STACK)).thenReturn(stack);

        assertEquals(trickState.getTrickSuit(), Card.Suit.NONE);

        List<Pair<UUID, Card>> stack2 = List.of(
                Pair.of(null, Card.BLUE_JESTER),
                Pair.of(null, Card.YELLOW_1),
                Pair.of(null, Card.BLUE_WIZARD),
                Pair.of(null, Card.RED_1),
                Pair.of(null, Card.GREEN_11)
        );
        TrickState trickState2 = mock(TrickState.class);
        when(trickState2.getTrickSuit()).thenCallRealMethod();
        when(trickState2.get(STACK)).thenReturn(stack2);

        assertEquals(trickState2.getTrickSuit(), Card.Suit.YELLOW);
    }
}
